@@ -208,6 +208,9 @@ class LevelDBBlockchain(Blockchain):
             else:
                 raise Exception("Database schema changed")
 
+    def GetStates(self, prefix, classref):
+        return DBCollection(self._db, None, prefix, classref)
+
     def GetAccountState(self, script_hash, print_all_accounts=False):
 
         if type(script_hash) is str:
@@ -408,6 +411,14 @@ class LevelDBBlockchain(Blockchain):
         logger.info("Could not find transaction for hash %s " % hash)
         return None, -1
 
+    def AddBlockDirectly(self, block):
+        if block.Index != self.Height + 1:
+            raise Exception("Invalid block")
+        if block.Index == len(self._header_index):
+            self.AddHeader(block.Header)
+        self.Persist(block)
+        self.OnPersistCompleted(block)
+
     def AddBlock(self, block):
 
         if not block.Hash.ToBytes() in self._block_cache:
