@@ -1,13 +1,12 @@
 from neo.Prompt.Commands.Invoke import InvokeContract, InvokeWithTokenVerificationScript
-from neo.Prompt.Utils import get_asset_id, get_from_addr
+from neo.Prompt.Utils import get_asset_id, get_from_addr, get_tx_attr_from_args
+from neo.Wallets.NEP5Token import NEP5Token
 from neocore.Fixed8 import Fixed8
 from prompt_toolkit import prompt
 from decimal import Decimal
-from neo.Wallets.utils import to_aes_key
 
 
 def token_send(wallet, args, prompt_passwd=True):
-
     if len(args) != 4:
         print("please provide a token symbol, from address, to address, and amount")
         return False
@@ -62,7 +61,6 @@ def token_send_from(wallet, args, prompt_passwd=True):
 
 
 def token_approve_allowance(wallet, args, prompt_passwd=True):
-
     if len(args) != 4:
         print("please provide a token symbol, from address, to address, and amount")
         return False
@@ -96,7 +94,6 @@ def token_approve_allowance(wallet, args, prompt_passwd=True):
 
 
 def token_get_allowance(wallet, args, verbose=False):
-
     if len(args) != 3:
         print("please provide a token symbol, from address, to address")
         return
@@ -121,16 +118,15 @@ def token_get_allowance(wallet, args, verbose=False):
 
 
 def token_mint(wallet, args, prompt_passwd=True):
-
     token = get_asset_id(wallet, args[0])
     mint_to_addr = args[1]
-
+    args, invoke_attrs = get_tx_attr_from_args(args)
     if len(args) < 3:
         raise Exception("please specify assets to attach")
 
     asset_attachments = args[2:]
 
-    tx, fee, results = token.Mint(wallet, mint_to_addr, asset_attachments)
+    tx, fee, results = token.Mint(wallet, mint_to_addr, asset_attachments, invoke_attrs=invoke_attrs)
 
     if results[0] is not None:
         print("\n-----------------------------------------------------------")
@@ -145,7 +141,7 @@ def token_mint(wallet, args, prompt_passwd=True):
                 print("incorrect password")
                 return
 
-        return InvokeWithTokenVerificationScript(wallet, tx, token, fee)
+        return InvokeWithTokenVerificationScript(wallet, tx, token, fee, invoke_attrs=invoke_attrs)
 
     else:
         print("Could not register addresses: %s " % str(results[0]))
@@ -186,7 +182,6 @@ def token_crowdsale_register(wallet, args, prompt_passwd=True):
 
 
 def do_token_transfer(token, wallet, from_address, to_address, amount, prompt_passwd=True):
-
     if from_address is None:
         print("Please specify --from-addr={addr} to send NEP5 tokens")
         return False
@@ -214,16 +209,47 @@ def do_token_transfer(token, wallet, from_address, to_address, amount, prompt_pa
     return False
 
 
-def amount_from_string(token, amount_str):
+def token_history(wallet, db, args):
+    if len(args) < 1:
+        print("Please provide the token symbol of the token you wish consult")
+        return False
 
+    if not db:
+        print("Unable to fetch history, notification database not enabled")
+        return False
+
+    token = get_asset_id(wallet, args[0])
+
+    if not isinstance(token, NEP5Token):
+        print("The given symbol does not represent a loaded NEP5 token")
+        return False
+
+    events = db.get_by_contract(token.ScriptHash)
+
+    addresses = wallet.Addresses
+    print("-----------------------------------------------------------")
+    print("Recent transaction history (last = more recent):")
+    for event in events:
+        if event.Type != 'transfer':
+            continue
+        if event.AddressFrom in addresses:
+            print(f"[{event.AddressFrom}]: Sent {string_from_amount(token, event.Amount)}"
+                  f" {args[0]} to {event.AddressTo}")
+        if event.AddressTo in addresses:
+            print(f"[{event.AddressTo}]: Received {string_from_amount(token, event.Amount)}"
+                  f" {args[0]} from {event.AddressFrom}")
+    print("-----------------------------------------------------------")
+    return True
+
+
+def amount_from_string(token, amount_str):
     precision_mult = pow(10, token.decimals)
-    amount = float(amount_str) * precision_mult
+    amount = Decimal(amount_str) * precision_mult
 
     return int(amount)
 
 
 def string_from_amount(token, amount):
-
     precision_mult = pow(10, token.decimals)
     amount = Decimal(amount) / Decimal(precision_mult)
     formatter_str = '.%sf' % token.decimals
