@@ -222,37 +222,37 @@ class ApplicationEngine(ExecutionEngine):
                     self.gas_consumed = self.gas_consumed + (self.GetPrice() * self.ratio)
                 #                print("gas consumeb: %s " % self.gas_consumed)
                 except Exception as e:
-                    logger.error("Exception calculating gas consumed %s " % e)
+                    logger.debug("Exception calculating gas consumed %s " % e)
                     self._VMState |= VMState.FAULT
                     return False
 
                 if not self.testMode and self.gas_consumed > self.gas_amount:
-                    logger.error("NOT ENOUGH GAS")
+                    logger.debug("NOT ENOUGH GAS")
                     self._VMState |= VMState.FAULT
                     return False
 
                 if not self.CheckItemSize():
-                    logger.error("ITEM SIZE TOO BIG")
+                    logger.debug("ITEM SIZE TOO BIG")
                     self._VMState |= VMState.FAULT
                     return False
 
                 if not self.CheckStackSize():
-                    logger.error("STACK SIZE TOO BIG")
+                    logger.debug("STACK SIZE TOO BIG")
                     self._VMState |= VMState.FAULT
                     return False
 
                 if not self.CheckArraySize():
-                    logger.error("ARRAY SIZE TOO BIG")
+                    logger.debug("ARRAY SIZE TOO BIG")
                     self._VMState |= VMState.FAULT
                     return False
 
                 if not self.CheckInvocationStack():
-                    logger.error("INVOCATION SIZE TO BIIG")
+                    logger.debug("INVOCATION SIZE TO BIIG")
                     self._VMState |= VMState.FAULT
                     return False
 
                 if not self.CheckDynamicInvoke():
-                    logger.error("Dynamic invoke without proper contract")
+                    logger.debug("Dynamic invoke without proper contract")
                     self._VMState |= VMState.FAULT
                     return False
 
