@@ -38,19 +38,26 @@ class StateMachine(StateReader):
         self._storages = storages
         self._wb = wb
 
-        self.Register("Neo.Account.SetVotes", self.Account_SetVotes)
-        self.Register("Neo.Validator.Register", self.Validator_Register)
+        # Standard Library
+        self.Register("System.Contract.GetStorageContext", self.Contract_GetStorageContext)
+        self.Register("System.Contract.Destroy", self.Contract_Destroy)
+        self.Register("System.Storage.Put", self.Storage_Put)
+        self.Register("System.Storage.Delete", self.Storage_Delete)
+
+        # Neo specific
         self.Register("Neo.Asset.Create", self.Asset_Create)
         self.Register("Neo.Asset.Renew", self.Asset_Renew)
-        self.Register("Neo.Contract.Create", self.Contract_Create)
         self.Register("Neo.Contract.Migrate", self.Contract_Migrate)
+        self.Register("Neo.Contract.Create", self.Contract_Create)
+
+        # Old
         self.Register("Neo.Contract.GetStorageContext", self.Contract_GetStorageContext)
         self.Register("Neo.Contract.Destroy", self.Contract_Destroy)
         self.Register("Neo.Storage.Put", self.Storage_Put)
         self.Register("Neo.Storage.Delete", self.Storage_Delete)
 
-        self.Register("AntShares.Account.SetVotes", self.Account_SetVotes)
-        self.Register("AntShares.Validator.Register", self.Validator_Register)
+        # Very old
+        self.Register("AntShares.Account.SetVotes", self.Deprecated_Method)
         self.Register("AntShares.Asset.Create", self.Asset_Create)
         self.Register("AntShares.Asset.Renew", self.Asset_Renew)
         self.Register("AntShares.Contract.Create", self.Contract_Create)
@@ -89,87 +96,8 @@ class StateMachine(StateReader):
         if self._storages.DebugStorage:
             self._storages.Commit(self._wb, False)
 
-    def Blockchain_GetAccount(self, engine):
-        hash = UInt160(data=engine.EvaluationStack.Pop().GetByteArray())
-        address = Crypto.ToAddress(hash).encode('utf-8')
-
-        account = self._accounts.TryGet(address)
-
-        if account:
-            engine.EvaluationStack.PushT(StackItem.FromInterface(account))
-        else:
-            engine.EvaluationStack.PushT(False)
-
-        return True
-
-    def Blockchain_GetAsset(self, engine):
-
-        hash = UInt256(data=engine.EvaluationStack.Pop().GetByteArray())
-
-        asset = self._assets.TryGet(hash.ToBytes())
-        if asset:
-            engine.EvaluationStack.PushT(StackItem.FromInterface(asset))
-        else:
-            engine.EvaluationStack.PushT(False)
-        return True
-
-    def Blockchain_GetContract(self, engine):
-        hash = UInt160(data=engine.EvaluationStack.Pop().GetByteArray())
-
-        contract = self._contracts.TryGet(hash.ToBytes())
-
-        if contract:
-            engine.EvaluationStack.PushT(StackItem.FromInterface(contract))
-        else:
-            engine.EvaluationStack.PushT(False)
-        return True
-
-    def Account_SetVotes(self, engine):
-
-        try:
-            account = engine.EvaluationStack.Pop().GetInterface()
-
-            vote_list = engine.EvaluationStack.Pop().GetArray()
-        except Exception as e:
-            logger.error("could not get account or votes: %s " % e)
-            return False
-
-        if account is None or len(vote_list) > 1024:
-            return False
-
-        if account.IsFrozen:
-            return False
-
-        balance = account.BalanceFor(Blockchain.SystemShare().Hash)
-
-        if balance == Fixed8.Zero() and len(vote_list) > 0:
-            return False
-
-        acct = self._accounts.GetAndChange(account.AddressBytes)
-        voteset = []
-        for v in vote_list:
-            if v not in voteset:
-                voteset.append(v.GetByteArray())
-        acct.Votes = voteset
-
-        # print("*****************************************************")
-        # print("SET ACCOUNT VOTES %s " % json.dumps(acct.ToJson(), indent=4))
-        # print("*****************************************************")
-        return True
-
-    def Validator_Register(self, engine):
-
-        pubkey = ECDSA.decode_secp256r1(engine.EvaluationStack.Pop().GetByteArray(), unhex=False, check_on_curve=True).G
-        if pubkey.IsInfinity:
-            return False
-
-        if not self.CheckWitnessPubkey(engine, pubkey):
-            return False
-
-        vstate = ValidatorState(pub_key=pubkey)
-        validator = self._validators.GetOrAdd(pubkey.ToBytes(), vstate)
-        engine.EvaluationStack.PushT(StackItem.FromInterface(validator))
-        return True
+    def Deprecated_Method(self, engine):
+        logger.debug("Method No Longer operational")
 
     def Asset_Create(self, engine):
 
@@ -230,7 +158,7 @@ class StateMachine(StateReader):
             expiration=Blockchain.Default().Height + 1 + 2000000, is_frozen=False
         )
 
-        asset = self._assets.GetOrAdd(tx.Hash.ToBytes(), new_asset)
+        asset = self._assets.ReplaceOrAdd(tx.Hash.ToBytes(), new_asset)
 
         # print("*****************************************************")
         # print("CREATED ASSET %s " % tx.Hash.ToBytes())
@@ -439,53 +367,6 @@ class StateMachine(StateReader):
                                test_mode=engine.testMode))
         return True
 
-    def Storage_Get(self, engine):
-
-        context = None
-        try:
-            item = engine.EvaluationStack.Pop()
-            context = item.GetInterface()
-        except Exception as e:
-            logger.error("could not get storage context %s " % e)
-            return False
-
-        if not self.CheckStorageContext(context):
-            return False
-
-        key = engine.EvaluationStack.Pop().GetByteArray()
-        storage_key = StorageKey(script_hash=context.ScriptHash, key=key)
-        item = self._storages.TryGet(storage_key.ToArray())
-
-        keystr = key
-
-        valStr = bytearray(0)
-
-        if item is not None:
-            valStr = bytearray(item.Value)
-
-        if len(key) == 20:
-            keystr = Crypto.ToAddress(UInt160(data=key))
-
-            try:
-                valStr = int.from_bytes(valStr, 'little')
-            except Exception as e:
-                pass
-
-        if item is not None:
-
-            engine.EvaluationStack.PushT(bytearray(item.Value))
-
-        else:
-            engine.EvaluationStack.PushT(bytearray(0))
-
-        self.events_to_dispatch.append(
-            SmartContractEvent(SmartContractEvent.STORAGE_GET, ['%s -> %s' % (keystr, valStr)],
-                               context.ScriptHash, Blockchain.Default().Height + 1,
-                               engine.ScriptContainer.Hash if engine.ScriptContainer else None,
-                               test_mode=engine.testMode))
-
-        return True
-
     def Storage_Put(self, engine):
 
         context = None
@@ -507,7 +388,7 @@ class StateMachine(StateReader):
 
         new_item = StorageItem(value=value)
         storage_key = StorageKey(script_hash=context.ScriptHash, key=key)
-        item = self._storages.GetOrAdd(storage_key.ToArray(), new_item)
+        item = self._storages.ReplaceOrAdd(storage_key.ToArray(), new_item)
 
         keystr = key
         valStr = bytearray(item.Value)
