@@ -1,7 +1,6 @@
 from neo.Core.Blockchain import Blockchain
 from neo.Implementations.Blockchains.LevelDB.LevelDBBlockchain import LevelDBBlockchain
 import shutil
-import time
 from neo.Utils.NeoTestCase import NeoTestCase
 from neo.Settings import settings
 import os
@@ -16,7 +15,7 @@ class VerifiableTestCase(NeoTestCase):
     @classmethod
     def setUpClass(self):
         os.makedirs(self.LEVELDB_TESTPATH, exist_ok=True)
-        self._blockchain = LevelDBBlockchain(path=self.LEVELDB_TESTPATH)
+        self._blockchain = LevelDBBlockchain(path=self.LEVELDB_TESTPATH, skip_version_check=True)
         Blockchain.RegisterBlockchain(self._blockchain)
 
     @classmethod
