@@ -43,20 +43,11 @@ from neo.Prompt.Commands.Wallet import DeleteAddress, ImportWatchAddr, ImportTok
 
 from neo.Prompt.Utils import get_arg, get_from_addr, get_tx_attr_from_args, get_owners_from_params
 from neo.Prompt.InputParser import InputParser
-from neo.Settings import settings, PrivnetConnectionError, PATH_USER_DATA
+from neo.Settings import settings, PrivnetConnectionError
 from neo.UserPreferences import preferences
 from neocore.KeyPair import KeyPair
 from neocore.UInt256 import UInt256
 
-# Logfile settings & setup
-LOGFILE_FN = os.path.join(PATH_USER_DATA, 'prompt.log')
-LOGFILE_MAX_BYTES = 5e7  # 50 MB
-LOGFILE_BACKUP_COUNT = 3  # 3 logfiles history
-settings.set_logfile(LOGFILE_FN, LOGFILE_MAX_BYTES, LOGFILE_BACKUP_COUNT)
-
-# Prompt history filename
-FILENAME_PROMPT_HISTORY = os.path.join(PATH_USER_DATA, '.prompt.py.history')
-
 
 class PromptFileHistory(FileHistory):
     def append(self, string):
@@ -157,13 +148,16 @@ class PromptInterface:
                 'debugstorage {on/off/reset}'
                 ]
 
-    history = PromptFileHistory(FILENAME_PROMPT_HISTORY)
+    history = None
 
     token_style = None
     start_height = None
     start_dt = None
 
-    def __init__(self):
+    def __init__(self, history_filename=None):
+        if history_filename:
+            self.history = PromptFileHistory(history_filename)
+
         self.input_parser = InputParser()
         self.start_height = Blockchain.Default().Height
         self.start_dt = datetime.datetime.utcnow()
@@ -909,20 +903,6 @@ class PromptInterface:
             else:
                 print("Cannot configure VM instruction logging. Please specify on|off")
 
-        elif what == 'maxpeers':
-            try:
-                c1 = int(get_arg(args, 1).lower())
-                num_peers = int(c1)
-                if num_peers > 0:
-                    old_max_peers = settings.CONNECTED_PEER_MAX
-                    settings.set_max_peers(num_peers)
-                    NodeLeader.Instance().OnUpdatedMaxPeers(old_max_peers, num_peers)
-                    print("set max peers to %s " % num_peers)
-                else:
-                    print("Please specify integer greater than zero")
-            except Exception as e:
-                print("Cannot configure max peers. Please specify an integer greater than 0")
-
         else:
             print(
                 "Cannot configure %s try 'config sc-events on|off', 'config debug on|off', 'config sc-debug-notify on|off' or 'config vm-log on|off'" % what)
@@ -1085,6 +1065,12 @@ def main():
     elif args.coznet:
         settings.setup_coznet()
 
+    # Logfile settings & setup
+    logfile_fn = os.path.join(settings.DATA_DIR_PATH, 'prompt.log')
+    logfile_max_bytes = 5e7  # 50 MB
+    logfile_backup_count = 3  # 3 logfiles history
+    settings.set_logfile(logfile_fn, logfile_max_bytes, logfile_backup_count)
+
     if args.theme:
         preferences.set_theme(args.theme)
 
@@ -1103,7 +1089,8 @@ def main():
         NotificationDB.instance().start()
 
     # Start the prompt interface
-    cli = PromptInterface()
+    fn_prompt_history = os.path.join(settings.DATA_DIR_PATH, '.prompt.py.history')
+    cli = PromptInterface(fn_prompt_history)
 
     # Run things
 #    reactor.suggestThreadPoolSize(15)
